/**
 * Program skeleton for the course "Programming embedded systems"
 */

#include <stdio.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "stm32f10x_conf.h"
#include "task.h"
#include "setup.h"
#include "assert.h"

/*-----------------------------------------------------------*/

typedef s32 FIA;

#define P 1            // places after comma

#define FROM_FLOAT(x)  (FIA)((x) * (1 << P))
#define FADD(x, y)     ((x) + (y))
#define FSUB(x, y)     ((x) - (y))
#define FMUL(x, y)     (((x) * (y)) >> P)
#define FDIV(x, y)     (((x) << P) / (y))

void print(FIA x) {
  printf("%f\n", ((double)x) / (1 << P));
}

void demo(void *params) {
  FIA x, y;

  printf("Using P = %d\n", P);

  x = FROM_FLOAT(1.5);
  y = FROM_FLOAT(3.0);
  printf("x = "); print(x);
  printf("y = "); print(y);
  printf("FMUL(x, y) = "); print(FMUL(x, y));
  assert(FDIV(FMUL(x, y), y) == x);
  printf("FMUL(FROM_FLOAT(1.5), FROM_FLOAT(0.5)) = "); print(FMUL(FROM_FLOAT(1.5), FROM_FLOAT(0.5)));

  vTaskDelay(portMAX_DELAY);
}

/*
 * Entry point of program execution
 */
int main( void )
{
  prvSetupHardware();

  xTaskCreate(demo, "demo", 100, NULL, 1, NULL);

  vTaskStartScheduler();
  assert(0);
  return 0;                 // not reachable
}
/*-----------------------------------------------------------*/


